//+--------------------------------------------------------------------+
//| Copyright:  (C) 2014, Miroslav Popov - All rights reserved!        |
//| Website:    http://forexsb.com/                                    |
//| Support:    http://forexsb.com/forum/                              |
//| License:    Proprietary under the following circumstances:         |
//|                                                                    |
//| This code is a part of Forex Strategy Builder. It is free for      |
//| use as an integral part of Forex Strategy Builder.                 |
//| One can modify it in order to improve the code or to fit it for    |
//| personal use. This code or any part of it cannot be used in        |
//| another applications without a permission. Contact information     |
//| cannot be changed.                                                 |
//|                                                                    |
//| NO LIABILITY FOR CONSEQUENTIAL DAMAGES                             |
//|                                                                    |
//| In no event shall the author be liable for any damages whatsoever  |
//| (including, without limitation, incidental, direct, indirect and   |
//| consequential damages, damages for loss of business profits,       |
//| business interruption, loss of business information, or other      |
//| pecuniary loss) arising out of the use or inability to use this    |
//| product, even if advised of the possibility of such damages.       |
//+--------------------------------------------------------------------+

#property copyright "Copyright 2014, Miroslav Popov"
#property link      "http://forexsb.com"
#property version   "1.00"
#property strict

#include <Forexsb.com/Indicator.mqh>
#include <Forexsb.com/Enumerations.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class PriceActionIndicatorOscillator : public Indicator
  {
public:
                     PriceActionIndicatorOscillator(SlotTypes slotType)
     {
      SlotType=slotType;

      IndicatorName="Price Action Indicator Oscillator";

      WarningMessage    = "";
      IsAllowLTF        = true;
      ExecTime          = ExecutionTime_DuringTheBar;
      IsSeparateChart   = true;
      IsDiscreteValues  = false;
      IsDeafultGroupAll = false;
     }

   virtual void      Calculate(DataSet &dataSet);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void PriceActionIndicatorOscillator::Calculate(DataSet &dataSet)
  {
   Data=GetPointer(dataSet);


// Reading the parameters
   int           Period=(int)NumParam[0].Value;
   double        dLevel= NumParam[1].Value;
   MAMethod  maMethod  =(MAMethod)ListParam[3].Index;
   int           iPrvs=CheckParam[0].Checked ? 1 : 0;

// Calculation
   double DispBuffer[];  ArrayResize(DispBuffer,Data.Bars);ArrayInitialize(DispBuffer, 0);

   int      iFirstBar=2;

   for(int iBar=iFirstBar; iBar<Data.Bars; iBar++)
      DispBuffer[iBar]=((Data.Close[iBar]-Data.Open[iBar])+(Data.Close[iBar]-Data.High[iBar])+(Data.Close[iBar]-Data.Low[iBar]))/2;

   double adMA[];   MovingAverage(Period,0,maMethod,DispBuffer,adMA);

// Saving the components

   ArrayResize(Component[0].Value,Data.Bars);
   Component[0].CompName  = "Fast line";
   Component[0].DataType  = IndComponentType_IndicatorValue;
   Component[0].FirstBar  = iFirstBar;
   ArrayCopy(Component[0].Value,DispBuffer);

   ArrayResize(Component[3].Value,Data.Bars);
   Component[3].CompName   = "Slow line";
   Component[3].DataType   = IndComponentType_IndicatorValue;
   Component[3].FirstBar   = iFirstBar;
   ArrayCopy(Component[3].Value,adMA);

   ArrayResize(Component[1].Value,Data.Bars);
   Component[1].FirstBar=iFirstBar;

   ArrayResize(Component[2].Value,Data.Bars);
   Component[2].FirstBar=iFirstBar;

// Sets the Component's type
   if(SlotType==SlotTypes_OpenFilter)
     {
      Component[1].DataType = IndComponentType_AllowOpenLong;
      Component[1].CompName = "Is long entry allowed";
      Component[2].DataType = IndComponentType_AllowOpenShort;
      Component[2].CompName = "Is short entry allowed";
     }
   else if(SlotType==SlotTypes_CloseFilter)
     {
      Component[1].DataType = IndComponentType_ForceCloseLong;
      Component[1].CompName = "Close out long position";
      Component[2].DataType = IndComponentType_ForceCloseShort;
      Component[2].CompName = "Close out short position";
     }

// Calculation of the logic
   IndicatorLogic indLogic=IndicatorLogic_It_does_not_act_as_a_filter;

   if(ListParam[0].Text=="The fast line rises")
     {
      indLogic=IndicatorLogic_The_indicator_rises;
     }
   else if(ListParam[0].Text=="The fast line falls")
     {
      indLogic=IndicatorLogic_The_indicator_falls;
     }
   else if(ListParam[0].Text=="The fast line is higher than the level line")
     {
      indLogic=IndicatorLogic_The_indicator_is_higher_than_the_level_line;
     }
   else if(ListParam[0].Text=="The fast line is lower than the level line")
     {
      indLogic=IndicatorLogic_The_indicator_is_lower_than_the_level_line;
     }
   else if(ListParam[0].Text=="The fast line crosses the level line upward")
     {
      indLogic=IndicatorLogic_The_indicator_crosses_the_level_line_upward;
     }
   else if(ListParam[0].Text=="The fast line crosses the level line downward")
     {
      indLogic=IndicatorLogic_The_indicator_crosses_the_level_line_downward;
     }
   else if(ListParam[0].Text=="The fast line changes its direction upward")
     {
      indLogic=IndicatorLogic_The_indicator_changes_its_direction_upward;
     }
   else if(ListParam[0].Text=="The fast line changes its direction downward")
     {
      indLogic=IndicatorLogic_The_indicator_changes_its_direction_downward;
     }
   else if(ListParam[0].Text=="The fast line crosses the slow line upward")
     {
      IndicatorCrossesAnotherIndicatorUpwardLogic(iFirstBar,iPrvs,DispBuffer,adMA,Component[1],
                                                  Component[2]);
     }
   else if(ListParam[0].Text=="The fast line crosses the slow line downward")
     {
      IndicatorCrossesAnotherIndicatorDownwardLogic(iFirstBar,iPrvs,DispBuffer,adMA,Component[1],
                                                    Component[2]);
     }
   else if(ListParam[0].Text=="The fast line is higher than the slow line")
     {
      IndicatorIsHigherThanAnotherIndicatorLogic(iFirstBar,iPrvs,DispBuffer,adMA,Component[1],
                                                 Component[2]);
     }
   else if(ListParam[0].Text=="The fast line is lower than the slow line")
     {
      IndicatorIsLowerThanAnotherIndicatorLogic(iFirstBar,iPrvs,DispBuffer,adMA,Component[1],
                                                Component[2]);
     }
   else if(ListParam[0].Text=="The slow line rises")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,0,0,Component[1],Component[2],
                      IndicatorLogic_The_indicator_rises);
     }
   else if(ListParam[0].Text=="The slow line falls")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,0,0,Component[1],Component[2],
                      IndicatorLogic_The_indicator_falls);
     }
   else if(ListParam[0].Text=="The slow line changes its direction upward")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,0,0,Component[1],Component[2],
                      IndicatorLogic_The_indicator_changes_its_direction_upward);
     }
   else if(ListParam[0].Text=="The slow line changes its direction downward")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,0,0,Component[1],Component[2],
                      IndicatorLogic_The_indicator_changes_its_direction_downward);
     }
   else if(ListParam[0].Text=="The slow line is higher than the level line")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,dLevel,0-dLevel,Component[1],Component[2],
                      IndicatorLogic_The_indicator_is_higher_than_the_level_line);
     }
   else if(ListParam[0].Text=="The slow line is lower than the level line")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,dLevel,0-dLevel,Component[1],Component[2],
                      IndicatorLogic_The_indicator_is_lower_than_the_level_line);
     }
   else if(ListParam[0].Text=="The slow line crosses the level line upward")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,dLevel,0-dLevel,Component[1],Component[2],
                      IndicatorLogic_The_indicator_crosses_the_level_line_upward);
     }
   else if(ListParam[0].Text=="The slow line crosses the level line downward")
     {
      OscillatorLogic(iFirstBar,iPrvs,adMA,dLevel,0-dLevel,Component[1],Component[2],
                      IndicatorLogic_The_indicator_crosses_the_level_line_downward);
     }

   OscillatorLogic(iFirstBar,iPrvs,DispBuffer,dLevel,0-dLevel,Component[1],Component[2],indLogic);

  }
//+------------------------------------------------------------------+
